header {
/*
 * ============================================================================
 * Licensed Materials - Property of IBM
 * Project  Zero
 *
 * (C) Copyright IBM Corp. 2007  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ============================================================================
 */
 
package zero.json.antlr;
}


/* #################################################################### */
/* #                              Lexer                                 */
/* #################################################################### */
class JsonLexer extends Lexer;

options {
	k=3;
	charVocabulary = '\0'..'\ufffe';  // uffff is EOF
	defaultErrorHandler=false;
}

tokens {
	Object;
	Array;
}


String 
	: ('"' ( ~('\u0000'..'\u001f' | '\\' | '\"' ) | Escapes )* '"') {
		String quotedString = $getText; 
		String s = quotedString.substring(1, quotedString.length()-1);
		$setText(s);
	};
	
protected Escapes
   	: "\\\""  { $setText("\""); }
   	| "\\\\"  { $setText("\\"); }
   	| "\\/"   { $setText("/");  }
   	| "\\b"   { $setText("\b"); }
   	| "\\f"   { $setText("\f"); }
   	| "\\n"   { $setText("\n"); }
   	| "\\r"   { $setText("\r"); }
   	| "\\t"   { $setText("\t"); }
   	| ("\\u" HexDigit HexDigit HexDigit HexDigit) { 
   		String s = $getText;
		char codeValue = (char) Integer.parseInt(s.substring(2), 16);
		$setText(codeValue);
	}
   	;

protected HexDigit 
	: '0'..'9' | 'A'..'F' | 'a'..'f';

Number 
	: (('-')? ('0' | '1'..'9' (Digit)*) ('.' (Digit)*)? (('e'|'E') ('+'|'-')? (Digit)+)?);

protected Digit 
	: '0'..'9';
	
True 
	: "true";
	
False 
	: "false";

Null 
	: "null";

LBrace
	options { paraphrase = "'{'"; }
	: '{';
	
RBrace
	options { paraphrase = "'}'"; }
	: '}';
	
LBracket
	options { paraphrase = "'['"; }
	: '[';

RBracket
	options { paraphrase = "']'"; }
	: ']';
	
Comma
	options { paraphrase = "','"; }
	: ',';
	
Colon
	options { paraphrase = "':'"; }
	: ':';
	
	
WS 
	: ( ' ' 
		| '\t' { tab(); } 
		| ( '\r' ( '\n' )? | '\n' ) { newline(); }
	) { $setType(Token.SKIP); }
	;


/* #################################################################### */
/* #                             Parser                                 */
/* #################################################################### */
{
import antlr.TokenStreamSelector;

import java.io.StringReader;
}

class JsonParser extends Parser("zero.antlr.Parser");


options {
	buildAST=true; 	  // Automatically build the AST while parsing
}


value
	: String | Number | True | False | Null | object | array
	;
	
object
	: ((LBrace! (ws)* (pair (ws)* (Comma! (ws)* pair (ws)*)*)? RBrace!))
		{ #object = #([Object, "object"], #object); }
	;
	
pair
	: String (ws)* Colon! (ws)* value
	;
	
array
	: ((LBracket! (ws)* (value (ws)* (Comma! (ws)* value (ws)*)*)? RBracket!))
		{ #array = #([Array, "array"], #array); }
	;
	
ws
	: WS
	;
	
	
// 'exactlyOneValue' ensures exactly one JSON value in the stream.
exactlyOneValue
	: value EOF!
	;
	
optionalobject
	: object | EOF!
	;
	
	
/* #################################################################### */
/* #                             Walker                                 */
/* #################################################################### */
{
import java.io.File;
import java.io.FileNotFoundException;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.Map;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.Stack;

import zero.json.JsonArrayCallback;
import zero.json.JsonObjectCallback;
import zero.json.JsonParserCallback;
import zero.json.internal.StandardJsonParserCallback;
import zero.json.internal.StringFilterRegistry;

import zero.core.logging.LogHelper;
}

class JsonTreeWalker extends TreeParser;


{
private JsonParserCallback parserCB = new StandardJsonParserCallback();
private StringFilterRegistry stringFilterRegistry = null;
private Stack<String> keyStack = null;
public JsonTreeWalker(JsonParserCallback cb) {
	this();
	parserCB = cb;
	if (cb.requireNamedObjects()) {
		keyStack = new Stack<String>();
	}
}

/**
 * Set the registry to be used for invoking JSON string filters.
 * Used from the Json API.
 */
public void setStringFilterRegistry(StringFilterRegistry filterRegistry) {
	stringFilterRegistry = filterRegistry;
}

@Override
public void reportError(RecognitionException ex) {
	throw new RuntimeException(ex);
}
}


parse returns [Object v = null]
	: n:Number {
		try {
			long l = Long.parseLong(n.getText());
			v = l;

		} catch (NumberFormatException x) {
			double d = Double.parseDouble(n.getText());
			v = d;
		} }
	| s:String {
		v = (null == stringFilterRegistry) ? s.getText() : stringFilterRegistry.decodeString(s.getText()); }
	| t:True {
		v = Boolean.parseBoolean(t.getText()); }
	| f:False {
		v = Boolean.parseBoolean(f.getText()); }
	| u:Null {
		v = null; }
	| o:Object {
		Map<String,Object> objectMap = null;
		String keyForObjectCreation = null;
		// Peek at the key if any from the stack and pass it as an argument.
		if (null != keyStack && !keyStack.isEmpty()) {
			keyForObjectCreation = keyStack.peek();
		}
		JsonObjectCallback objectCB = parserCB.createJsonObject(keyForObjectCreation);
		if (null == objectCB) {
			objectMap = new HashMap<String,Object>();
		}
		AST child = o.getFirstChild();
		while (child != null) {
			String key = (String)parse(child);
			// push the key on to the stack so that any object or array creation can use it
			if (null != keyStack) {
				keyStack.push(key);
			}
			child = child.getNextSibling();
			Object value = parse(child);
			// pop the key off the stack
			if (null != keyStack) {
				keyStack.pop();
			}
			if (null != objectCB) {
				objectCB.put(key, value);
			} else {
				objectMap.put(key, value);
			}
			child = child.getNextSibling();
		}
		if (null != objectCB)
			v = objectCB.getObject();
		else {
			v = objectMap;
		} }
	| a:Array {
		ArrayList<Object> arrayList = null;
		String keyForObjectCreation = null;
		if (null != keyStack) {
			keyForObjectCreation = keyStack.peek();
		}
		JsonArrayCallback arrayCB = parserCB.createJsonArray(keyForObjectCreation);
		if (null == arrayCB) {
			arrayList = new ArrayList<Object>();
		}
		for (AST child = a.getFirstChild(); child != null; child = child.getNextSibling()) {
			if (null != arrayCB) {
				arrayCB.add(parse(child));
			} else {
				arrayList.add(parse(child));
			}
		}
		if (null != arrayCB) {
			v = arrayCB.getObject();
		} else {
			v = arrayList;
		} }
	;
